"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.reactFrameworkPlugin = void 0;
const path_1 = __importDefault(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const component_loader_plugin_1 = require("./component-loader-plugin");
const extract_component_1 = require("./extract-component");
const optimize_deps_plugin_1 = require("./optimize-deps-plugin");
const react_imports_plugin_1 = require("./react-imports-plugin");
const svgr_plugin_1 = require("./svgr-plugin");
exports.reactFrameworkPlugin = {
    isCompatible: async (dependencies) => {
        const react = dependencies["react"];
        if (!react) {
            return false;
        }
        return react.majorVersion >= 16;
    },
    async create({ svgr } = {}) {
        const previewDirPath = path_1.default.resolve(__dirname, "..", "preview");
        return {
            name: "@previewjs/plugin-react",
            defaultWrapperPath: "__previewjs__/Wrapper.tsx",
            previewDirPath,
            tsCompilerOptions: {
                typeRoots: [path_1.default.join(previewDirPath, "node_modules", "@types")],
                jsx: typescript_1.default.JsxEmit.ReactJSX,
                jsxImportSource: "react",
            },
            componentDetector: (program, filePaths) => {
                const components = [];
                for (const filePath of filePaths) {
                    components.push(...(0, extract_component_1.extractReactComponents)(program, filePath));
                }
                return components;
            },
            viteConfig: (config) => {
                return {
                    plugins: [
                        (0, optimize_deps_plugin_1.optimizeReactDepsPlugin)(),
                        (0, component_loader_plugin_1.reactComponentLoaderPlugin)({
                            config,
                        }),
                        (0, svgr_plugin_1.svgrPlugin)({
                            exportedComponentName: (svgr === null || svgr === void 0 ? void 0 : svgr.componentName) || "ReactComponent",
                            alias: config.alias,
                        }),
                        (0, react_imports_plugin_1.reactImportsPlugin)(),
                    ],
                    define: {
                        "process.env.RUNNING_INSIDE_PREVIEWJS": "1",
                    },
                };
            },
        };
    },
};
